import numpy as np
import xarray as xr

try:
    from best_estimate_common import BestEstimate
    from var_names import *
except ImportError:
    from .best_estimate_common import BestEstimate
    from .var_names import *

class BestEstimateDownShortHemisp(BestEstimate):
    def __init__(self, direct:np.ndarray, qc_direct:np.ndarray,
        diffuse:np.ndarray, qc_diffuse:np.ndarray, zenith_radians:np.ndarray):
        super().__init__()

        self.direct_out = direct
        self.qc_direct_out = qc_direct
        self.diffuse_out = diffuse
        self.qc_diffuse_out = qc_diffuse

        self.zenith_radians = zenith_radians
        self.varname = down_short_hemisp

    def calc_best_estimate(self):
        bad_val_mask = np.logical_or(np.asarray(self.qc_direct_out > 0), 
            np.asarray(self.qc_diffuse_out > 0))

        self.best_estimates = self.direct_out * np.cos(self.zenith_radians) + self.diffuse_out
        self.best_estimates[bad_val_mask] = self.missing_value
        self.qc_best_estimates = np.zeros(self.best_estimates.shape, dtype=np.int32)
        self.qc_best_estimates[bad_val_mask] = self.qc_missing_value

        del self.zenith_radians

    def set_output_vars(self, ds_out:xr.Dataset):
        ds_out[down_short_hemisp].data = BestEstimate.to_output_interval(self.best_estimates)
        ds_out['qc_' + down_short_hemisp].data = BestEstimate.to_output_interval(
            self.qc_best_estimates)

    # parent class modifies self.data_flag_out, which DownShortHemisp doesn't have
    def apply_min_and_max(self) -> None:
        var_data = self.best_estimates
        #qc_data = self.qc_best_estimates

        bad_value_mask = np.logical_or(np.asarray(var_data > self.maximum),
            np.asarray(var_data < self.minimum))
        var_data[bad_value_mask] = self.missing_value
        #qc_data[bad_value_mask] |= self.qc_missing_value