import numpy as np
import xarray as xr

from pdb import set_trace

try:
    from .BestEstimate2Instruments import BestEstimate2Instruments, BestEstimate
    from .var_names import up_short_hemisp
except ImportError:
    from BestEstimate2Instruments import BestEstimate2Instruments, BestEstimate
    from var_names import up_short_hemisp

class BestEstimateUpShortHemisp(BestEstimate2Instruments):
    def __init__(self):
        super().__init__()
        self.varname = up_short_hemisp

        self.low_zenith_max_diff_percent = 0.2
        self.high_zenith_max_diff = 5.0
        self.high_zenith_max_diff_percent = 0.1
        self.low_zenith_mask:np.ndarray = None

    def set_qc_thresholds(self, zenith_degrees:np.ndarray) -> None:
        self.low_zenith_mask = np.asarray(zenith_degrees < 80)

    def get_input_fields(self) -> tuple:
        if self.input_data_dicts is None:
            self.input_data_dicts = {}
            self.input_data_dicts['sirsC'] = BestEstimate.get_data_qc_dict(
                self.sirsC, up_short_hemisp, 'sirsC')
            self.input_data_dicts['sirsE'] = BestEstimate.get_data_qc_dict(
                self.sirsE, up_short_hemisp, 'sirsE')

        if self.data_flag_weekly is None:
            self.data_flag_weekly = self.get_weekly_flag(up_short_hemisp)

        return self.input_data_dicts, self.data_flag_weekly

    def should_check_last_best_estimate(self, val_sirsE:float, 
        val_sirsC:float,
        offset:int, data_available_flag:int) -> bool:
        # If only one instrument is available
        if val_sirsC is None:
            return True

        # SUGGESTED CHANGE: instrument_diff should be the abs of the diff,
        # like the 3-inst vars do
        self.data_flag_out[offset] = self.SOURCE_1_2
        self.best_estimates[offset] = (val_sirsE + val_sirsC) / 2
        self.instrument_diffs[offset] = val_sirsE - val_sirsC
        diff_1_2 = abs(self.instrument_diffs[offset])

        if self.low_zenith_mask[offset]:
            return (val_sirsE == 0 or 
            not abs(val_sirsC / val_sirsE - 1) < self.low_zenith_max_diff_percent)
        else:
            return not (diff_1_2 < self.high_zenith_max_diff or 
                self.best_estimates[offset] == 0 or
                diff_1_2 / self.best_estimates[offset] < self.high_zenith_max_diff_percent)

    # Does not override BestEstimate2Instruments::check_last_best_estimate