# Calculate values for albedo and net_surface_radiation
import numpy as np
from pdb import set_trace

try:
    from best_estimate_common import BestEstimate
except ImportError:
    from .best_estimate_common import BestEstimate

def calc_albedo(up_short_hemisp:np.ndarray,
    down_short_hemisp:np.ndarray, zenith_degrees:np.ndarray) -> np.ndarray:
    bad_val_mask = (np.asarray(up_short_hemisp <= 0) | 
        np.asarray(down_short_hemisp <= 0) | np.asarray(zenith_degrees > 93))
    # It's OK if this trips a divide by zero,
    # since that value is about to go missing anyway
    albedo = up_short_hemisp / down_short_hemisp
    # any division by zero in the above line is erased here
    albedo[bad_val_mask] = BestEstimate.missing_value
    return BestEstimate.to_output_interval(albedo)

def calc_net_surface_radiation(up_short_hemisp:np.ndarray, 
    down_short_hemisp:np.ndarray, up_long_hemisp:np.ndarray,
    down_long_hemisp:np.ndarray) -> np.ndarray:
    failure_threshold = BestEstimate.missing_value + 1E-10
    bad_val_mask = (np.asarray(up_short_hemisp < failure_threshold) |
        np.asarray(down_short_hemisp < failure_threshold) |
        np.asarray(up_long_hemisp < failure_threshold) |
        np.asarray(down_long_hemisp < failure_threshold))

    net_surface_radiation = (down_short_hemisp - up_short_hemisp) +\
        (down_long_hemisp - up_long_hemisp)
    net_surface_radiation[bad_val_mask] = BestEstimate.missing_value
    return BestEstimate.to_output_interval(net_surface_radiation)
