# This file contains the instrument availability and selection functions
# that are not specific to a certain BestEstimate class
# (i.e. those that don't use source flags besides SOURCE_NONE or FLAG_INSUFFICIENT_DATA)

import numpy as np
import xarray as xr

from pdb import set_trace

# Data flag vars: which variables were used in the estimate?
SOURCE_NONE = -4

FLAG_INSUFFICIENT_DATA = 4

# Which instrument data is available?
AVAILABLE_ALL = 10
AVAILABLE_NONE = -4
AVAILABLE_1 = -50
AVAILABLE_2 = -60
AVAILABLE_3 = -70
AVAILABLE_1_2 = -20
AVAILABLE_1_3 = -40
AVAILABLE_2_3 = -30
INSTR_AVAILABLE_QC_FAILED = -80

AVAILABLE_FLAGS = [AVAILABLE_1, AVAILABLE_1_2, AVAILABLE_1_3,
    AVAILABLE_2, AVAILABLE_2_3, AVAILABLE_3, AVAILABLE_3]

def num_instruments_available(availability_flag:int, num_sources:int=3) -> int:
    if availability_flag == AVAILABLE_ALL:
        return num_sources
    elif (availability_flag == AVAILABLE_1 or 
        availability_flag == AVAILABLE_2 or availability_flag == AVAILABLE_3):
        return 1
    return 2

def get_flag_data_available(qc_bad_1:int, qc_bad_2:int, qc_bad_3:int,
    qc_missing_value:int) -> int:
    qc_bad_1 &= qc_missing_value
    qc_bad_2 &= qc_missing_value
    qc_bad_3 &= qc_missing_value
    
    if qc_bad_1 and qc_bad_2 and qc_bad_3:
        return AVAILABLE_NONE
    elif not (qc_bad_1 or qc_bad_2 or qc_bad_3):
        return AVAILABLE_ALL
    elif not qc_bad_1:
        if not qc_bad_2:
            return AVAILABLE_1_2
        elif not qc_bad_3:
            return AVAILABLE_1_3
        else:
            return AVAILABLE_1
    elif not qc_bad_2:
        if not qc_bad_3:
            return AVAILABLE_2_3
        else:
            return AVAILABLE_2
    return AVAILABLE_3