import xarray as xr
import numpy as np
import csv
from datetime import date

from pdb import set_trace

try:
    from best_estimate_common import BestEstimate
    from var_names import *
except ImportError:
    from .best_estimate_common import BestEstimate
    from .var_names import *

TIMESTAMP_LENGTH = len('2024-06-05T23:53:00')

def add_space_padding(string:str) -> str:
    return string + (' ' * (TIMESTAMP_LENGTH - len(string)))

def add_var_data(output_data:dict, dataset_names:list,
    varnames_to_best_estimates:dict, all_missing:np.ndarray) -> None:
    for dataset_name in dataset_names:
        dataset_name_lower = dataset_name.lower()
        for varname in varnames_to_best_estimates:
            outvar_name = add_space_padding(dataset_name_lower + '_' + varname)
            var_data = varnames_to_best_estimates[varname]
            if dataset_name in var_data:
                output_data[outvar_name] = var_data[dataset_name]['data']

def add_time_data(output_data:dict, datasets:tuple) -> None:
    for dataset in datasets:
        if dataset is not None:
            output_data[add_space_padding('time')] = dataset['time'].data
            return

def save_raw_data(best_estimate_objects:dict,
    time_in:np.ndarray, zenith_degrees:np.ndarray, begin_date:int) -> None:
    for varname in best_estimate_objects:
        try:
            best_estimate_objects[varname].get_input_fields()
        except: # DownShortHemisp doesn't have this method, which is fine
            pass

    all_missing = BestEstimate.placeholder_var_data['data']

    dirn = best_estimate_objects[short_direct_normal].input_data_dicts
    dif = best_estimate_objects[down_short_diffuse_hemisp].input_data_dicts
    long = best_estimate_objects[down_long_hemisp].input_data_dicts
    up = best_estimate_objects[up_short_hemisp].input_data_dicts
    long_up = best_estimate_objects[up_long_hemisp].input_data_dicts

    varnames_to_best_estimates = {
           'dirn' : dirn,
            'dif' : dif,
           'long' : long,
             'up' : up,
        'long_up' : long_up
    }

    output_data = {
          add_space_padding('time') : time_in,
        add_space_padding('zenith') : zenith_degrees
    }
    #output_data[add_space_padding('zenith')] = zenith_degrees
    add_var_data(output_data, ('sirsE', 'sirsC', 'brs'), varnames_to_best_estimates, 
        all_missing)

    fieldnames = list(output_data.keys())
    # We handle "time" separately
    time_key = fieldnames[0]
    run_date = date.fromtimestamp(begin_date)
    outfile_path = f'{run_date.year}{run_date.month:02d}{run_date.day:02d}_raw.data'

    with open(outfile_path, 'w') as outfile:
        writer = csv.DictWriter(outfile, fieldnames, delimiter=' ')
        writer.writeheader()

        #set_trace()
        for i in range(BestEstimate.len_input_arrays):
            row = { time_key : time_in[i].astype('datetime64[s]')}
            row |= { key : f'{output_data[key][i]:.6f}'.ljust(TIMESTAMP_LENGTH) 
                for key in fieldnames if key != time_key}
            writer.writerow(row)

    return f'Wrote output file {outfile_path}'