"""A setuptools-based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject

To install locally during development (from this dir):
$> pip3 install --user -e .
"""

from pathlib import Path
from setuptools import setup
from subprocess import check_output, DEVNULL, CalledProcessError
from os import getenv
from codecs import open
import os
import re

proctype = 'vap'
process  = 'beflux1long'
package  = process
execname = process


def parse_requirement(line: str) -> str:
    line = line.strip()
    if line.startswith("#"):
        return ""
    elif "#" in line:
        return line[: line.find("#")].strip()
    return line


# Get the full package name
name = getenv('BUILD_PACKAGE_NAME', proctype+'-'+package)

# Get package version
version = getenv('BUILD_PACKAGE_VERSION')
if not version:
    command = './package/get_package_version.sh'
    version = check_output(command.split()).decode('utf-8').strip()

# Write the version.py file
with open(package+'/version.py', 'w') as f:
    f.write('# This file is autogenerated at build time.\n')
    f.write('# It should not be committed to the repository.\n')
    f.write('__version__ = "{}"'.format(name+'-'+version))

# Fix version to make Python happy
version = version.replace('-', '.', 1)     # change x.y-z to x.y.z
version = re.sub('\.el\d+$', '', version);   # remove Linux build platform (i.e. .el#)
version = re.sub('\.dirty$', '', version); # remove .dirty

# Get URL of git repository
command = 'git config --get remote.origin.url'
try:
    result = check_output(command.split(), stderr=DEVNULL)
    url = result.decode('utf-8').strip()
except CalledProcessError:
    url = ''

# Install pip requirements from requirements.txt, if it exists.
requirements = []
req_file = Path("requirements.txt")
if req_file.exists():
    lines = req_file.read_text().splitlines()
    for line in lines:
        parsed = parse_requirement(line)
        if parsed:
            requirements.append(parsed)

setup(
    name         = name,
    version      = version,
    url          = url,
    author       = 'None',
    author_email = 'None',
    packages     = [package],
    install_requires=requirements,
    entry_points={
        'console_scripts': [
            '{} = {}.__main__:main'.format(execname, package),
        ],
    },
)
